// by Abeno no Ossan is licensed under the Creative Commons - Public Domain Dedication license.
//                                              2017/04/17
#include <Servo.h>
#include <avr/pgmspace.h>

#define VERSION		2.1		// ver2.1
#define BUFF		127		// Buffer Size
#define SBUFF		 32		// Buffer Size

#define SHIFT		 7		// Operation 128
#define R			 0		// Red LED
#define G			 1		// Green LED
#define B			 2		// Blue LED
#define LED			 3		// LED
#define TIME		15		// Column of Time
#define MAXSN		12		// Max Number of Servos
#define MAXMN		21		// Max Number of Motions
#define MAXFN		 8		// Max Number of Frames 
#define POWER		17		// Servo power supply control pin
#define ERR			-1		// Error
#define DELAYTIME	500		// Delay Time
#define BITRATE	  57600		// Bow Rate
#define S_MAX	MAXSN-1		// S_Maxnumber
#define A_MAX		180		// A Maxvalu
#define RGBT_MAX	255		// RGBT Maxvalu

void nextFrame( void );			// Motion Play
void nextPose( void );			// Make a pose
int getPose( void );			// get buffer values of the next pose from serial data
void printThreeDigit( int );	// Print OUT ASCII Three-digit
void printTwoDigit( int );		// Print OUT ASCII Two-digit
int readThreeDigit( int );		// Read ASCII Three-digit
int readTwoDigit( int );		// Read ASCII Two-digit
int readOneDigit( void );		// Read ASCII One-digit


Servo servo[MAXSN];
unsigned char eyes[LED] = { 0, 0, 0 };

// Fine angle adjustments (degrees)
PROGMEM const int Trim[MAXSN] = {
		 0,		// Head yaw
		 0,		// Waist yaw
		 0,		// R Sholder roll
		 0,		// R Sholder pitch
		 0,		// R Hand grip
		 0,		// L Sholder roll
		 0,		// L Sholder pitch
		 0,		// L Hand grip
		 0,		// R Foot yaw
		 0,		// R Foot pitch
		 0,		// L Foot yaw
		 0 		// L Foot pitch
};

int NowAngle[MAXSN] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };				// Initialize array to 0
int TargetAngle[MAXSN] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };			// Initialize array to 0
int DeltaAngle[MAXSN] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };				// Initialize array to 0
unsigned char bufferAngle[MAXSN] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };	// Initialize array to 0
unsigned char tempAngle[MAXSN];

int NowBright[LED] = { 0, 0, 0 };				// Initialize array to 0
int TargetBright[LED] = { 0, 0, 0 };			// Initialize array to 0
int DeltaBright[LED] = { 0, 0, 0 };				// Initialize array to 0
unsigned char bufferBright[LED] = { 0, 0, 0 };	// Initialize array to 0
unsigned char tempBright[LED];

unsigned long startTime = 0;		// Motion start time(ms)
unsigned long endTime = 0;			// Motion end time(ms)
unsigned char bufferTime = 0;		// Motion buffer time (100ms)

unsigned char motionNumber = 0;
unsigned char frameNumber = 0;

char mode = 'M';

PROGMEM const unsigned char motion[MAXMN][MAXFN][TIME+1] = {
// M00 Stop
//         Right       Left        Right   Left     LED        Time
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,255,  5},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M01 Forward
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0, 90, 90,180, 90, 90, 80,110, 80,120,  0,  0,  0,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 70, 90, 70, 90,  0,  0,255,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 70, 70, 70, 80,  0,  0,255,  5},
  { 90, 90,  0, 90, 90,180, 90, 90,100, 60,100, 70,  0,  0,  0,  5},
  { 90, 90,  0, 90, 90,180, 90, 90,110, 90,110, 90,  0,  0,255,  5},
  { 90, 90,  0, 90, 90,180, 90, 90,110,100,110,110,  0,  0,255,  5},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M02 Back
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0, 90, 90,180, 90, 90,100,110,100,120,  0,  0,  0,  5},
  { 90, 90,  0, 90, 90,180, 90, 90,110, 90,110, 90,  0,  0,255,  5},
  { 90, 90,  0, 90, 90,180, 90, 90,110, 70,110, 80,  0,  0,255,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 80, 30, 80, 70,  0,  0,  0,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 70, 90, 70, 90,  0,  0,255,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 70,100, 70,110,  0,  0,255,  5},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M03 Head shake
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 30, 90,  0,130,120,180, 50, 70, 90, 90, 90, 90,  0,  0, 10, 10},
  {150, 90,  0,130, 70,180, 50,120, 90, 90, 90, 90,  0,  0,255, 10},
  { 30, 90,  0,130,120,180, 50, 70, 90, 90, 90, 90,  0,  0, 10, 10},
  {150, 90,  0,130, 70,180, 50,120, 90, 90, 90, 90,  0,  0,255, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  5},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M04 Body twist
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 30,  0,130,120,180, 50, 70, 90, 90, 90, 90, 10,  0,  0, 10},
  { 90,150,  0,130, 70,180, 50,120, 90, 90, 90, 90,255,  0,  0, 10},
  { 90, 30,  0,130,120,180, 50, 70, 90, 90, 90, 90, 10,  0,  0, 10},
  { 90,150,  0,130, 70,180, 50,120, 90, 90, 90, 90,255,  0,  0, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  5},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M05 Right turn
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0, 90, 90,180, 90, 90, 95, 60, 85, 70,  0,  0,  0,  5},
  { 90, 90,  0, 90, 90,180, 90, 90,100, 90, 80, 90,  0,  0,255,  5},
  { 90, 90,  0, 90, 90,180, 90, 90,100,100, 80,110,  0,  0,  0,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 85,110, 95,120,  0,  0,255,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 80, 90,100, 90,  0,  0,  0,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 80, 70,100, 80,  0,  0,255,  5},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M06 Left turn
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0, 90, 90,180, 90, 90, 95,110, 85,120,  0,  0,  0,  5},
  { 90, 90,  0, 90, 90,180, 90, 90,100, 90, 80, 90,  0,  0,255,  5},
  { 90, 90,  0, 90, 90,180, 90, 90,100, 70, 80, 80,  0,  0,  0,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 85, 60, 95, 70,  0,  0,255,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 80, 90,100, 90,  0,  0,  0,  5},
  { 90, 90,  0, 90, 90,180, 90, 90, 80,100,100,110,  0,  0,255,  5},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M07 Right arm vertical updown
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0,115, 90,180, 50, 90, 90, 90, 90, 90,255,255,  0, 10},
  { 90, 90,180,115, 90,180, 50, 90, 90, 90, 90, 90,255,255,  0, 10},
  { 90, 90,  0,115, 90,180, 50, 90, 90, 90, 90, 90,  0,255,255, 10},
  { 90, 90,180,115, 90,180, 50, 90, 90, 90, 90, 90,  0,255,255, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M08 Left arm vertical updown
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0,130, 90,180, 65, 90, 90, 90, 90, 90,  0,255,255, 10},
  { 90, 90,  0,130, 90,  0, 65, 90, 90, 90, 90, 90,  0,255,255, 10},
  { 90, 90,  0,130, 90,180, 65, 90, 90, 90, 90, 90,255,  0,255, 10},
  { 90, 90,  0,130, 90,  0, 65, 90, 90, 90, 90, 90,255,  0,255, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M09 Double arm vertical updown
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0,115, 90,180, 65, 90, 90, 90, 90, 90,255,255,  0, 10},
  { 90, 90,180,115, 90,  0, 65, 90, 90, 90, 90, 90,255,255,  0, 10},
  { 90, 90,  0,115, 90,180, 65, 90, 90, 90, 90, 90,255,  0,255, 10},
  { 90, 90,180,115, 90,  0, 65, 90, 90, 90, 90, 90,255,  0,255, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M10 Double hand open and close
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90, 90,130, 90, 90, 50, 90, 90, 90, 90, 90,255,  0,  0,  5},
  { 90, 90, 90,130, 50, 90, 50,130, 90, 90, 90, 90,255,  0,  0,  5},
  { 90, 90, 90,130, 90, 90, 50, 90, 90, 90, 90, 90,  0,255,  0,  5},
  { 90, 90, 90,130, 50, 90, 50,130, 90, 90, 90, 90,  0,255,  0,  5},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M11 Right arm horizontal updown
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0, 30, 90,180, 50, 90, 90, 90, 90, 90,255,  0,  0, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,255,  0,  0, 10},
  { 90, 90,  0, 30, 90,180, 50, 90, 90, 90, 90, 90,  0,255,  0, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,255,  0, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M12 Left arm horizontal updown
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0,130, 90,180,150, 90, 90, 90, 90, 90,255,  0,  0, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,255,  0,  0, 10},
  { 90, 90,  0,130, 90,180,150, 90, 90, 90, 90, 90,  0,255,  0, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,255,  0, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M13 Double arm horizontal updown
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0, 30, 90,180,150, 90, 90, 90, 90, 90,255,  0,  0, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,255,  0,  0, 10},
  { 90, 90,  0, 30, 90,180,150, 90, 90, 90, 90, 90,  0,255,  0, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,255,  0, 10},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M14 stand on right leg
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,255, 10},
  { 90, 90,  0, 90, 90,180, 50, 90, 90,115, 90,130,255,  0,  0, 30},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M15 stand on left leg
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,255, 10},
  { 90, 90,  0,130, 90,180, 90, 90, 90, 50, 90, 65,255,  0,  0, 30},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M16 stand up hight
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0,115, 90,180, 65, 90, 90, 90, 90, 90,  0,  0,255,  5},
  { 90, 90,180,115, 90,  0, 65, 90, 90, 30, 90,150,  0,  0,255, 30},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M17 Right hand shake
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90,120,120,130, 50,180, 90, 90, 90, 90, 90, 90,255,255,  0,  7},
  { 90,120,120, 90, 90,180, 90, 90, 90, 90, 90, 90,255,255,  0,  7},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M18 Right hand push
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0, 90, 90,180, 90, 90, 90, 90, 90, 90,  0,  0,255,  5},
  { 90, 90, 90,130,110,180, 50, 90, 90, 90, 90, 90,  0,  0,  0, 10},
  { 90, 90, 90,130,110,180, 50, 90, 90, 90, 90, 90,  0,  0,255,  5},
  { 90, 90, 90,130,110,180, 50, 90, 90, 90, 90, 90,  0,  0,255, 25},
  { 90, 90, 90,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  5},
  { 40,140, 90, 70, 90,180, 90, 90, 90, 90, 90, 90,  0,  0,255, 10},
  { 40,140, 90, 70, 90,180, 90, 90, 90, 90, 90, 90,  0,  0,255, 25},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M19 Left hand push
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0, 50, 90,180, 90, 90, 90, 90, 90, 90,  0,  0,255,  5},
  { 90, 90,  0,130, 90, 90, 50,110, 90, 90, 90, 90,  0,  0,  0, 10},
  { 90, 90,  0,130, 90, 90, 50,110, 90, 90, 90, 90,  0,  0,255,  5},
  { 90, 90,  0,130, 90, 90, 50,110, 90, 90, 90, 90,  0,  0,255, 25},
  { 90, 90,  0,130, 90, 90, 50, 90, 90, 90, 90, 90,  0,  0,  0,  5},
  {140, 40,  0, 50, 90, 90,110, 90, 90, 90, 90, 90,  0,  0,255, 10},
  {140, 40,  0, 50, 90, 90,110, 90, 90, 90, 90, 90,  0,  0,255, 25},
  { 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,  0,  0}},
// M20 matthieu
// Had  Bd sho arm han sho arm han leg ank leg ank   R   G   B   T
 {{ 90, 90,  0,130, 90,180, 50, 90, 90, 90, 90, 90,  0,  0,255, 10},
  { 90, 90,  0,100, 90,180, 80, 90,100, 90, 80, 90,  0,255,  0, 10},
  { 90, 90,  0, 70, 90,180,110, 90, 90, 90, 90, 90,255,  0,  0, 10},
  { 90, 90,  0, 40, 90,180,140, 90,100, 90, 80, 90,  0,255,  0, 10},
  { 90, 90,  0, 10, 90,180,170, 90, 90, 90, 90, 90,  0,  0,255, 10},
  { 90, 90,  0, 40, 90,180,140, 90,100, 90, 80, 90,  0,255,  0, 10},
  { 90, 90,  0, 70, 90,180,110, 90, 90, 90, 90, 90,255,  0,  0, 10},
  { 90, 90,  0,100, 90,180, 80, 90,100, 90, 80, 90,  0,255,  0, 10}}
};


void setup(void)
{
	int i;

	servo[ 0].attach(10);		// Head yaw
	servo[ 1].attach(11);		// Waist yaw
	servo[ 2].attach( 9);		// R Sholder roll
	servo[ 3].attach( 8);		// R Sholder pitch
	servo[ 4].attach( 7);		// R Hand grip
	servo[ 5].attach(12);		// L Sholder roll
	servo[ 6].attach(13);		// L Sholder pitch
	servo[ 7].attach(14);		// L Hand grip
	servo[ 8].attach( 4);		// R Foot yaw
	servo[ 9].attach( 2);		// R Foot pitch
	servo[10].attach(15);		// L Foot yaw
	servo[11].attach(16);		// L Foot pitch
	eyes[R] = 6;				// Red LED of eyes
	eyes[G] = 5;				// Green LED of eyes
	eyes[B] = 3;				// Blue LED of eyes

	for( i = 0; i < MAXSN; i++ ) {
		TargetAngle[i] = (int)pgm_read_byte( &motion[0][0][i] ) << SHIFT;
		NowAngle[i] = TargetAngle[i];
		servo[i].write( (int)pgm_read_byte( &motion[0][0][i] ) + pgm_read_word( &Trim[i] ));
	}

	for( i = 0; i < LED; i++ ) {
		TargetBright[i] = 0;
		NowBright[i] = 0;
		analogWrite( eyes[i], 0 );
	}

	Serial.begin( BITRATE );

	delay( DELAYTIME );

	pinMode( POWER, OUTPUT );
	digitalWrite( POWER, LOW );
} 


void loop( void )
{
	int i;
	int buf = ERR;
	int RemainingTime = 0;			// Motion remaining time(msec)


	if( Serial.available( )) {
		if( Serial.read( ) == '#' ) {
			while( !Serial.available( )){}
			switch( Serial.read( )) {
				case 'C':
					Serial.print( "#C" );
					if( bufferTime > 0 ) {
						Serial.print( "F" );
					}
					else {
						Serial.print( "0" );
					}
					break;

				case 'P':
					buf = getPose( );
					if( buf != ERR ) {
						mode = 'P';
						digitalWrite( POWER, HIGH );
					}
					else {
						Serial.print( "#EP" );
					}
					break;

				case 'M':
					buf = readTwoDigit( MAXMN - 1 );
					if( buf != ERR ) {
						mode = 'M';
						motionNumber = buf;
						frameNumber = 0;
						digitalWrite( POWER, HIGH );
						Serial.print( "#M" );
						printTwoDigit( motionNumber );
					}
					else {
						Serial.print( "#EM" );
					}
					break;

				case 'S':		// Servo Stop
//					Serial.print( "#S" );
					digitalWrite( POWER, LOW );
					break;

				case 'Q':
					Serial.print( "#Q" );
					if( mode == 'M' ) {
						Serial.print( "M" );
						printTwoDigit( motionNumber );
						Serial.print( "T" );
						buf = (int)( endTime - millis( )) /100;
						if( buf < 0 ) buf = 0;
						printThreeDigit( buf );
					}
					if( mode == 'P' ) {
						Serial.print( "PT" );
						buf = (int)( endTime - millis( )) /100;
						if( buf < 0 ) buf = 0;
						printThreeDigit( buf );
					}
					break;

				case 'R':		// Servo Read
					Serial.print( "#R" );
					for( i = 0; i < MAXSN; i++ ) {
						Serial.print( "S" );
						printTwoDigit( i );
						Serial.print( "A" );
						buf = servo[i].read( );
//						buf = servo[i].read( ) - pgm_read_word( &Trim[i] );
						printThreeDigit( buf );
					}
					break;

				case 'V':
					Serial.print( "Ver " );
					Serial.print( VERSION );
					break;

				default:
					Serial.print( "#E" );
					break;
			}
			Serial.print("\n");		// add LF(0x0a)
		}
	}

	if( endTime > millis( ) ) {
		RemainingTime = (int)( endTime - millis( )) /10;
		for( i = 0; i < MAXSN; i++ ) {
			NowAngle[i] = TargetAngle[i] - ( DeltaAngle[i] * RemainingTime );
			servo[i].write(( NowAngle[i] >> SHIFT ) + pgm_read_word( &Trim[i] ));
		}
		for( i = 0; i < LED; i++ ) {
			NowBright[i] = TargetBright[i] - ( DeltaBright[i] * RemainingTime );
			analogWrite( eyes[i], NowBright[i] >> SHIFT );
		}
	}
	else if( mode == 'P' ) {
		if( bufferTime > 0 ) {
			nextPose( );
		}
//		else if(( endTime + ( unsigned long )DELAYTIME ) < millis( ) ) {
			//digitalWrite( POWER, LOW );
//		}
	}
	else if( mode == 'M' ) {
		nextFrame( );
	}
}


//Motion Play
void nextFrame( void )
{
	int i;


	if( frameNumber >= MAXFN ) {
		frameNumber = 0;
	}

	for( i = 0; i < MAXSN; i++ ) {
		bufferAngle[i] = pgm_read_byte( &motion[motionNumber][frameNumber][i] );
	}
	for( i = 0; i < LED; i++ ) {
		bufferBright[i] = pgm_read_byte( &motion[motionNumber][frameNumber][MAXSN+i] );
	}
	bufferTime = pgm_read_byte( &motion[motionNumber][frameNumber][TIME] );

	frameNumber++;
	nextPose( );
}


// Make a pose
void nextPose( void )
{
	int i;


	if( bufferTime > 0 ) {
		for( i = 0; i < MAXSN; i++ ) {
			TargetAngle[i] = (int)bufferAngle[i] << SHIFT;								// *128
			DeltaAngle[i] = ( TargetAngle[i] - NowAngle[i] ) / ( bufferTime * 10 );		// 128*deg/s
		}
		for( i = 0; i < LED; i++ ) {
			TargetBright[i] = (int)bufferBright[i] << SHIFT;							// *128
			DeltaBright[i] = ( TargetBright[i] - NowBright[i] ) / ( bufferTime * 10 );	// 128/s
		}
	}
	else {
		for( i = 0; i < MAXSN; i++ ) {
			DeltaAngle[i] = 0;
		}
		for( i = 0; i < LED; i++ ) {
			DeltaBright[i] = 0;
		}
	}
	startTime = millis( );
	endTime = startTime + (unsigned long)( bufferTime * 100 );	// ms
	bufferTime = 0;
}


// get buffer values of the next pose from serial data
int getPose( void )
{
	int i;
	int buf = 0;
	int value = 0;
	boolean readPose = true;


	if( bufferTime == 0 ) {
		//Initialize array to target angle
		for( i = 0; i < MAXSN; i++ ) {
			tempAngle[i] = bufferAngle[i];
		}
		for( i = 0; i < LED; i++ ) {
			tempBright[i] = bufferBright[i];
		}
	}
	else {
		readPose = false;
//		Serial.print( "#EP" );
		return ERR;
	}

	//Read data
	Serial.print( "#P" );
	while( readPose ) {
		while( !Serial.available( ) ) {}
		switch( Serial.read( ) ) {
			case 'S':
				value = readTwoDigit( S_MAX );
				Serial.print( "S" );
				if( value != ERR ) {
					printTwoDigit( value );
					while( !Serial.available( ) ) {}
					if( Serial.read( ) == 'A' ) {
						buf = readThreeDigit( A_MAX );
						if( buf != ERR ) {
							tempAngle[value] = buf;
							Serial.print( "A" );
							printThreeDigit( buf );
						}
						else {
							readPose = false;
						}
					}
				}
				break;

			case 'R':
				buf = readThreeDigit( RGBT_MAX );
				if( buf != ERR ) {
					tempBright[R] = buf;
					Serial.print( "R" );
					printThreeDigit( buf );
				}
				else {
					readPose = false;
				}
				break;

			case 'G':
				buf = readThreeDigit( RGBT_MAX );
				if( buf != ERR ) {
					tempBright[G] = buf;
					Serial.print( "G" );
					printThreeDigit( buf );
				}
				else {
					readPose = false;
				}
				break;

			case 'B':
				buf = readThreeDigit( RGBT_MAX );
				if( buf != ERR ) {
					tempBright[B] = buf;
					Serial.print( "B" );
					printThreeDigit( buf );
				}
				else {
					readPose = false;
				}
				break;

			case 'T':
				buf = readThreeDigit( RGBT_MAX );
				if( buf > 0 ) {
					bufferTime = buf;
					Serial.print( "T" );
					printThreeDigit( buf );

					for( i = 0; i < MAXSN; i++ ){
						bufferAngle[i] = tempAngle[i];
					}
					for( i = 0; i < LED; i++ ) {
						bufferBright[i] = tempBright[i];
					}
				}
				readPose = false;
				break;

			default:
				buf = ERR;
				readPose = false;
				break;
		}
	}
	return buf;
}


//Print OUT ASCII Three-digit
void printThreeDigit( int buf )
{
	String s = String( buf );
	if( s.length( ) == 2 ) {
		Serial.print( "0" );
	}
	else if ( s.length( ) == 1 ) {
		Serial.print( "00" );
	}
	Serial.print( s );
}


//Print OUT ASCII Two-digit
void printTwoDigit( int buf )
{
	String s = String( buf );
	if ( s.length( ) == 1 ) {
		Serial.print( "0" );
	}
	Serial.print( s );
}


//Read ASCII Three-digit
int readThreeDigit( int maximum )
{
	int digit;
	int buf;

	buf = readOneDigit( );
	if( buf != ERR ) {
		digit = buf * 100;
		buf = readOneDigit( );
		if( buf != ERR ) {
			digit += buf * 10;
			buf = readOneDigit( );
			if( buf != ERR ) {
				digit += buf;
				if( digit <= maximum ) {
					buf = digit;
				}
				else {
					buf = ERR;
				}
			}
		}
	}
	return buf;
}


//Read ASCII Two-digit
int readTwoDigit( int maximum )
{
	int digit;
	int buf;

	buf = readOneDigit( );
	if( buf != ERR ) {
		digit = buf * 10;
		buf = readOneDigit( );
		if( buf != ERR ) {
			digit += buf;
			if( digit <= maximum ) {
				buf = digit;
			}
			else {
				buf = ERR;
			}
		}
	}
	return buf;
}


//Read ASCII One-digit
int readOneDigit( void )
{
	int buf;

	while( !Serial.available( ) ) {}
	buf = Serial.read( ) - '0';
	if( buf < 0 || 9 < buf ) {
		buf = ERR;
	}
	return buf;
}
